// Copyright (C) 2018-2024 Intel Corporation
// SPDX-License-Identifier: Apache-2.0
//

#pragma once

#include "openvino/op/op.hpp"
#include "snippets/lowered/linear_ir.hpp"

namespace ov {
namespace snippets {
namespace op {

/**
 * @interface Kernel
 * @brief Generated by Canonicalization and represents compute kernel legal for scheduling
 * @ingroup snippets
 */
class Kernel : public ov::op::Op {
public:
    OPENVINO_OP("Kernel", "SnippetsOpset");

    Kernel(lowered::LinearIR region);
    Kernel() = default;

    lowered::LinearIR region;

    std::shared_ptr<Node> clone_with_new_inputs(const OutputVector& inputs) const override {
        return std::make_shared<Kernel>(region);
    }
    const void *compile_params = nullptr;
};

} // namespace op
} // namespace snippets
} // namespace ov
