# Copyright (C) 2018-2024 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import numpy as np
import pytest
import tensorflow as tf
from common.tf_layer_test_class import CommonTFLayerTest
from common.utils.tf_utils import mix_array_with_value

class TestMulNoNan(CommonTFLayerTest):
    def _prepare_input(self, inputs_info):
        assert 'x' in inputs_info
        assert 'y' in inputs_info
        x_shape = inputs_info['x']
        y_shape = inputs_info['y']
        inputs_data = {}
        inputs_data['x'] = np.random.randint(-10, 10, x_shape).astype(self.input_type)
        inputs_data['x'] = mix_array_with_value(inputs_data['x'], np.inf)
        inputs_data['x'] = mix_array_with_value(inputs_data['x'], np.nan)
        inputs_data['y'] = np.random.randint(-10, 10, y_shape).astype(self.input_type) * \
                           np.random.choice([0.0], y_shape).astype(self.input_type)
        return inputs_data

    def create_mul_no_nan_net(self, input_shape, input_type):
        self.input_type = input_type
        tf.compat.v1.reset_default_graph()
        with tf.compat.v1.Session() as sess:
            x = tf.compat.v1.placeholder(input_type, input_shape, 'x')
            y = tf.compat.v1.placeholder(input_type, input_shape, 'y')
            tf.raw_ops.MulNoNan(x=x, y=y)
            tf.compat.v1.global_variables_initializer()
            tf_net = sess.graph_def
        return tf_net, None

    test_data_basic = [
        dict(input_shape=[10, 10], input_type=np.float32),
        dict(input_shape=[2, 3, 4], input_type=np.float32),
    ]

    @pytest.mark.parametrize("params", test_data_basic)
    @pytest.mark.precommit_tf_fe
    @pytest.mark.nightly
    def test_mul_no_nan_basic(self, params, ie_device, precision, ir_version, temp_dir,
                              use_new_frontend):
        self._test(*self.create_mul_no_nan_net(**params),
                   ie_device, precision, ir_version, temp_dir=temp_dir,
                   use_new_frontend=use_new_frontend)